/*
 *  mod_musicindex.c
 *  mod_musicindex
 *
 *  $Id: mod_musicindex.c,v 1.31 2003/06/25 14:35:51 varenet Exp $
 *
 *  Created by Regis BOUDIN on Sat Dec 28 2002.
 *  Copyright (c) 2003 Regis BOUDIN
 *  Copyright (c) 2003 Thibaut VARENE
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */


/**
 * @file 
 * Core file.
 *
 * This file is the core of the module. It contains Apache's mandatory stuff.
 *
 * @author Regis Boudin
 * @author Thibaut Varene
 * @version MUSIC_VERSION_STRING
 * @date 2003
 *
 * @bug Many many unknown bugs.
 *
 * @warning Use it at your own risks!
 *
 * @todo MORE DOCUMENTATION OF THE CODE!!!
 * @todo check data types (no more 'int', 'short'/'long' are our friends.
 * @todo caching ?
 * @todo custom playlist handling accross several directories.
 * @todo tag rewriting (eventually, but if i get everything right, we're just
 * sending the raw file to the player, so that's not gonna be easy :P
 * @todo Considering the previous item, we might need a handler for
 * streaming, otherwise we won't be able to restrain file downloading
 * (Apache::MP3 doesn't do anything better there fwiw).
 * @todo Test behaviour with MP3 VBR files.
 * @todo eventually prepare the possibility to generate ices playlists.
 */

#include "mod_musicindex.h"
#include "playlist.h"
#include "config.h"
#include "html.h"
#include "inf.h"

struct mu_ent *new_ent(pool *pool, struct mu_ent *head)
{
	struct mu_ent *p = (struct mu_ent *) ap_pcalloc(pool, sizeof(struct mu_ent));
	memset(p, 0, sizeof(struct mu_ent));	
	p->next = head;
	return p;
}

struct mu_ent *quicksort(struct mu_ent *base, struct mu_ent *end,
		      mu_config * conf) 
{
	struct mu_ent *prem;
	struct mu_ent *deux = base;
	struct mu_ent *refe = base;

	if ((base == end) || (base->next == end))
		return base;

	prem = base->next;

	do {
		if (inf_global(prem, refe, conf) < 0) {
			deux->next = prem->next;
			prem->next = base;
			base = prem;
			prem = deux->next;
		} else {
			deux = prem;
			prem = prem->next;
		}
	} while (prem != end);

	base = quicksort(base, refe, conf);
	refe->next = quicksort(refe->next, end, conf);

	return base;
}

int handle_musicindex(request_rec * r)
{
	char *ifile;
	mu_config *conf = (mu_config *)
		ap_get_module_config(r->per_dir_config, &musicindex_module);

	r->allowed |= (1 << M_GET);
	if ((r->method_number != M_GET) || (!(conf->options & MI_ACTIVE)))
		return DECLINED;

	if (r->uri[0] == '\0' || r->uri[strlen(r->uri) - 1] != '/') {
		if (r->args != NULL)
			ifile = ap_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
		      "/", "?", r->args, NULL);
		else
			ifile = ap_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
		      "/", NULL);

		ap_table_setn(r->headers_out, "Location",
		ap_construct_url(r->pool, ifile, r));
		return HTTP_MOVED_PERMANENTLY;
	}

	switch (treat_args(r, conf)) {
		case 'A':
			return musicindex_directory(r, conf);
		case 'P':
			return playlist_directory(r, conf);
		case 'L':
			return playlist_selected(r, conf);
		default:
			return 0;
	}

}

const command_rec musicindex_cmds[] = {
	{"MusicSortOrder", sort_order, NULL, OR_INDEXES, RAW_ARGS,
		"can be : title album artist track length bitrate filetype"},
	{"MusicFields", set_fields, NULL, OR_INDEXES, RAW_ARGS,
		"can be : title album artist track length bitrate filetype"},
	{"MusicAllowStream", allow_stream, NULL, OR_INDEXES, NO_ARGS,
		"Enable streaming"},
	{"MusicAllowDownload", allow_download, NULL, OR_INDEXES, NO_ARGS,
		"Enable download"},
	{"MusicAllowSearch", allow_search, NULL, OR_INDEXES, NO_ARGS,
		"Enable searching (EXPERIMENTAL)"},
	{"MusicLister", music_lister, NULL, OR_INDEXES, NO_ARGS,
		"Enable the Musicindex maker"},
	{NULL}
};

const handler_rec musicindex_handlers[] = {
	{DIR_MAGIC_TYPE, handle_musicindex},
	{NULL}
};

module MODULE_VAR_EXPORT musicindex_module = {
	STANDARD_MODULE_STUFF,
	NULL,				/* initializer */
	create_musicindex_config,	/* dir config creater */
	merge_musicindex_configs,	/* dir merger */
	NULL,				/* server config */
	NULL,				/* merge server config */
	musicindex_cmds,		/* command table */
	musicindex_handlers,		/* handlers */
	NULL,				/* filename translation */
	NULL,				/* check_user_id */
	NULL,				/* check auth */
	NULL,				/* check access */
	NULL,				/* type_checker */
	NULL,				/* fixups */
	NULL,				/* logger */
	NULL,				/* header parser */
	NULL,				/* child_init */
	NULL,				/* child_exit */
	NULL				/* post read-request */
};
